﻿using System;

namespace vm.Aspects.Diagnostics.DumpImplementation
{
    struct DumpedProperty : IEquatable<DumpedProperty>
    {
        public object Instance;
        public string Property;

        public DumpedProperty(object instance, string property)
        {
            if (instance == null)
                throw new ArgumentNullException("instance");
            if (string.IsNullOrWhiteSpace(property))
                throw new ArgumentNullException("property");

            Instance = instance;
            Property = property;
        }

        #region Identity rules implementation.
        #region IEquatable<DumpedProperty> Members
        public bool Equals(DumpedProperty other)
        {
            return Instance.Equals(other.Instance) &&
                   Property.Equals(other.Property, StringComparison.OrdinalIgnoreCase);
        }
        #endregion

        public override bool Equals(object obj)
        {
            if (obj is DumpedProperty)
                return Equals((DumpedProperty)obj);

            return false;
        }

        public override int GetHashCode()
        {
            var hash = Constants.HashInitializer;

            hash = hash * Constants.HashMultiplier + Instance.GetHashCode();
            hash = hash * Constants.HashMultiplier + Property.GetHashCode();

            return hash;
        }

        public static bool operator==(DumpedProperty left, DumpedProperty right)
        {
            return left.Equals(right);
        }

        public static bool operator!=(DumpedProperty left, DumpedProperty right)
        {
            return !(left==right);
        }
        #endregion
    }
}
