﻿using System;
using System.Collections.Generic;
using System.Reflection;

namespace vm.Aspects.Diagnostics.DumpImplementation
{
    /// <summary>
    /// Compares two PropertyInfo objects by dump attribute's properties Order and then by Name.
    /// </summary>
    class PropertyDumpOrder : IComparer<PropertyInfo>
    {
        Type _metadata;

        public PropertyDumpOrder(Type metadata)
        {
            _metadata = metadata;
        }

        #region IComparer<PropertyInfo> Members

        public int Compare(PropertyInfo x, PropertyInfo y)
        {
            if (x == null)
                throw new ArgumentNullException("x");
            if (y == null)
                throw new ArgumentNullException("y");
            if (!x.DeclaringType.IsAssignableFrom(y.DeclaringType) &&
                !y.DeclaringType.IsAssignableFrom(x.DeclaringType))
                throw new InvalidOperationException("Cannot compare the orders of properties from unrelated classes.");

            // get the order of each property:
            var orderX = (long)PropertyDumpResolver.GetPropertyDumpAttribute(x, _metadata).Order;
            var orderY = (long)PropertyDumpResolver.GetPropertyDumpAttribute(y, _metadata).Order;

            // if the properties are from the same type
            if (x.DeclaringType == y.DeclaringType)
            {
                // negative orders should be dumped after the positive orders (and the base class' properties but this is not the case here)
                if (orderX < 0)
                    orderX = int.MaxValue + (-orderX);
                if (orderY < 0)
                    orderY = int.MaxValue + (-orderY);

                if (orderX < orderY)
                    return -1;

                if (orderX > orderY)
                    return 1;

                // if the orders are the same - dump in alphabetical order
                return string.CompareOrdinal(x.Name, y.Name);
            }

            // dump the properties of the base class after the non-negative properties of the inheriting class
            return y.DeclaringType.IsAssignableFrom(x.DeclaringType)
                        ? (orderX >= 0 ? -1 : 1)
                        : (orderY >= 0 ? 1 : -1);
        }

        #endregion
    }
}
